/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.knowledge.branch;

import ru.ispras.testbase.knowledge.basis.Generator;
import ru.ispras.testbase.knowledge.basis.GeneratorResult;
import ru.ispras.testbase.knowledge.integer.IntFormat;
import ru.ispras.testbase.knowledge.integer.IntNumber;

/**
 * {@link IfThenElseGenerator} is a base class for branch instructions' test data generators.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public abstract class IfThenElseGenerator implements Generator<IntNumber, IfThenElse> {
  /**
   * Specifies the number format.
   */
  public static final IntFormat FORMAT = IntFormat.INT32;

  /**
   * Minimal value of an operand (depends on #FORMAT).
   */
  public static final long MIN_VALUE = FORMAT.getMinValue();

  /**
   * Maximal value of an operand (depends on #FORMAT).
   */
  public static final long MAX_VALUE = FORMAT.getMaxValue();

  @Override
  public boolean check(
      final IfThenElse situation, final IntNumber[] operands) {
    return situation == IfThenElse.THEN ? checkThen(operands) : checkElse(operands);
  }

  @Override
  public GeneratorResult<IntNumber> generate(
      final IfThenElse situation, final IntNumber[] operands) {
    if (situation == IfThenElse.THEN) {
      final IntNumber[] result = generateThen(operands);
      return new GeneratorResult<>(checkThen(result), result);
    } else {
      final IntNumber[] result = generateElse(operands);
      return new GeneratorResult<>(checkElse(result), result);
    }
  }

  /**
   * Checks whether the operand values satisfy the branch condition.
   *
   * @param operands input values.
   * @return {@code true} iff satisfy.
   */
  public abstract boolean checkThen(final IntNumber[] operands);
  
  /**
   * Checks whether the operand values do not satisfy the branch condition.
   *
   * @param operands input values.
   * @return {@code true} iff do not satisfy.
   */
  public final boolean checkElse(final IntNumber[] operands) {
    return !checkThen(operands);
  }

  /**
   * Generates operand values that satisfy the branch condition.
   *
   * @param operands input values ({@code null} if unknown).
   * @return the generated values.
   */

  public abstract IntNumber[] generateThen(final IntNumber[] operands);
  
  /**
   * Generates operand values that violate the branch condition.
   *
   * @param operands input values ({@code null} if unknown).
   * @return the generated values.
   */
  public abstract IntNumber[] generateElse(final IntNumber[] operands);
}
